local utils = {}

local technologies = data.raw.technology

---@param name string
---@param prerequisites string[]?
function utils.set_prerequisites(name, prerequisites)
    local technology = technologies[name]
    technology.prerequisites = prerequisites
end

---@param name string
---@param prerequisites string[]
function utils.add_prerequisites(name, prerequisites)
    local technology = data.raw.technology[name]
    technology.prerequisites = technology.prerequisites or {}
    for _, prerequisite in pairs(prerequisites) do
        technology.prerequisites[#technology.prerequisites+1] = prerequisite
    end
end

---@param name string
---@param trigger data.TechnologyTrigger
function utils.set_trigger(name, trigger)
    local technology = technologies[name]
    technology.research_trigger = trigger
    technology.unit = nil
end

---@param name string
---@param unit data.TechnologyUnit
function utils.set_unit(name, unit)
    local technology = technologies[name]
    technology.unit = unit
    technology.research_trigger = nil
end

---@param name string
---@param recipes string[]
function utils.set_recipes(name, recipes)
    local technology = technologies[name]
    technology.effects = {}
    for i, effect in pairs(recipes) do
        technology.effects[i] = {
            type = "unlock-recipe",
            recipe = effect,
        }
    end
end

---@param name string
---@param recipes string[]
function utils.add_recipes(name, recipes)
    local technology = technologies[name]
    technology.effects = technology.effects or {}
    local len = #technology.effects
    for i = 1, #recipes do
        technology.effects[len + i] = {
            type = "unlock-recipe",
            recipe = recipes[i],
        }
    end
end

---@param name string
---@param recipes string[]
function utils.remove_recipes(name, recipes)
    local map = {}
    for _, recipe in pairs(recipes) do
        map[recipe] = true
    end
    local effects = technologies[name].effects --[[@as TechnologyModifier]]
    for i = #effects, 1, -1 do
        if map[effects[i].recipe] then
            table.remove(effects, i)
        end
    end
end

function utils.insert_recipe(name, recipe, position)
    local technology = technologies[name]
    technology.effects = technology.effects or {}
    table.insert(technology.effects, position, {
        type = "unlock-recipe",
        recipe = recipe,
    })
end

---@param name string
---@param packs string[]
function utils.remove_packs(name, packs)
    local map = {}
    for _, pack in pairs(packs) do
        map[pack] = true
    end
    local ingredients = technologies[name].unit.ingredients
    for i = #ingredients, 1, -1 do
        if map[ingredients[i][1]] then
            table.remove(ingredients, i)
        end
    end
end

return utils